/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Rene Wagner <rene.wagner@dfki.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __FEATURE_EXTRACTION_CONFIG_HPP__
#define __FEATURE_EXTRACTION_CONFIG_HPP__

//#define VSLAM_USE_SIFT 1
//#define VSLAM_USE_FAST 1
//#define VSLAM_USE_STAR 1
//#define VSLAM_USE_STAR_CALONDER 1
//#define VSLAM_USE_FAST_CALONDER 1

#include <vslam/features/sift_extractor.hpp>
#include <vslam/features/fast_surf_extractor.hpp>
#include <vslam/features/star_surf_extractor.hpp>
#include <vslam/features/fast_calonder_extractor.hpp>
#include <vslam/features/star_calonder_extractor.hpp>
#include <vslam/features/feature_extractor.hpp>

namespace feature_extraction_config {
	
#ifdef VSLAM_USE_SIFT
	typedef vslam::features::sift_extractor feature_extractor;
#elif VSLAM_USE_FAST
	typedef vslam::features::fast_surf_extractor feature_extractor;
#elif VSLAM_USE_STAR
	typedef vslam::features::star_surf_extractor feature_extractor;
#elif VSLAM_USE_FAST_CALONDER
	typedef vslam::features::fast_calonder_extractor feature_extractor;
#elif VSLAM_USE_STAR_CALONDER
	typedef vslam::features::star_calonder_extractor feature_extractor;
#else // SURF
	typedef vslam::features::feature_extractor feature_extractor;
#endif

	enum
	{
		descriptor_size = feature_extractor::descriptor_size
	};
	
} // namespace feature_extraction_config

#endif // __FEATURE_EXTRACTION_CONFIG_HPP__
