/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Oliver Birbach <oliver.birbach@dfki.de>
 *          Rene Wagner <rene.wagner@dfki.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#include "plugin_dump.h"
#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>

#include "program_config.h"

PluginDump::PluginDump(std::string _name, Buffer * _buffer, Buffer * _other_buffer,
                       const vslam::optimizer::pose_list &trajectory, const vslam::map::map &global_map,
                       const program_config &pc)
    : ProcessingPlugin(_name, _buffer), other_buffer(_other_buffer),
	  trajectory_(trajectory),
	  global_map_(global_map),
	  pc_(pc)
{
}


PluginDump::~PluginDump()
{
}

Result PluginDump::process(Frame * data)
{
    // Only camera 0 should do some stuff
    if (data->cam_id == 0)
    {
        // Get other Frame from other camera
        //Frame *other_data = other_buffer->getPointer(other_buffer->curWrite());

		std::stringstream suffix;
		suffix << std::setfill('0') << std::setw(4) << data->number << ".dat";

		// FIXME precision etc.
		if (pc_.dump_landmarks)
		{
			std::stringstream fname;
			fname << pc_.dump_dir << "/landmarks-" << suffix.str();
			std::ofstream f(fname.str().c_str());
			
			const vslam::map::map &map = global_map_;

			for (vslam::map::map::landmarks_list::const_iterator i = map.get_landmarks().begin();
				 i != map.get_landmarks().end(); ++i)
			{
				f << i->position->x() << " " << i->position->y() << " " << i->position->z() << std::endl;
			}
			f.flush();
			f.close();
		}

		if (pc_.dump_trajectory)
		{
			std::stringstream fname;
			fname << pc_.dump_dir << "/trajectory-" << suffix.str();
			std::ofstream f(fname.str().c_str());

			f.precision(20);
			
			for (vslam::optimizer::pose_list::const_iterator i = trajectory_.begin();
				 i != trajectory_.end(); ++i)
			{
				f << (**i).orient.x() << " " << (**i).orient.y() << " " << (**i).orient.z() << " " << (**i).orient.w() << " "
				  << (**i).pos.x() << " " << (**i).pos.y() << " " << (**i).pos.z() << " "
				  << (**i).vel.x() << " " << (**i).vel.y() << " " << (**i).vel.z()
				  << std::endl;
			}
			f.flush();
			f.close();
		}
    }    

    return Ok;  
}

bool PluginDump::event(SDL_Event &event)
{
    bool event_handled = false;
    
    return event_handled;
}
