/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Oliver Birbach <oliver.birbach@dfki.de>
 *          Rene Wagner <rene.wagner@dfki.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#include "plugin_extract_features.h"
#include <iostream>

PluginExtractFeatures::PluginExtractFeatures(std::string _name, Buffer * _buffer, Buffer * _other_buffer, int feature_limit)
    : ProcessingPlugin(_name, _buffer), other_buffer(_other_buffer), feature_limit(feature_limit)
{
}


PluginExtractFeatures::~PluginExtractFeatures()
{
}

Result PluginExtractFeatures::process(Frame * data)
{
	// restart timer
	data->t.restart();

	timeutils::timer t;
	cv::Mat img(data->video.p);

	unsigned num_keypoints; // original number of keypoints
	extract_(img, data->features, feature_limit, &num_keypoints);

	fprintf(stderr, "extraction: %s %s/%s %lld %llu %u %u\n", data->cam_id == 0 ? "left" : "right", extract_.detector_id(), extract_.descriptor_id(), 
			data->number, t.elapsed(), num_keypoints, data->features.size());
	//std::cout << "[" << data->number << "] features " << (data->cam_id == 0 ? "left" : "right") << ": " << data->features.size() << std::endl;
	
    return Ok;  
}

bool PluginExtractFeatures::event(SDL_Event &event)
{
    bool event_handled = false;
    
    return event_handled;
}
