/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Oliver Birbach <oliver.birbach@dfki.de>
 *          Rene Wagner <rene.wagner@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#include "plugin_imu.h"
#include <iostream>
#include "frame.h"

PluginIMU::PluginIMU(std::string _name, Buffer * _buffer, imu_raw_packets_rb_t & _rb, double _imu_lag)
  : ProcessingPlugin(_name, _buffer), rb(_rb), imu_lag(_imu_lag), logfile("imu_timestamps.dat")
{   
  logfile << "# physical_event imu_lag_compensated\n";
  logfile.precision(14);
}

PluginIMU::~PluginIMU()
{
}

Result PluginIMU::process(Frame * data)
{
    if (data->cam_id == 0)
    {    
        data->imu_measurements.clear();

        IMURDataPacket p;
        double imu_lag_compensated;        
        double physical_event;

        while(rb.size() > 0)
        {
            rb.pop(p);
            
            // Append to data log for saving data
            data->imu_measurement_log.append(p);
            
            imu_lag_compensated = (p.m_time.tv_sec + p.m_time.tv_nsec / 1e9) - imu_lag;
            physical_event = (data->physical_event_ts.tv_sec + data->physical_event_ts.tv_nsec / 1e9);
            if (physical_event >= imu_lag_compensated)
                data->imu_measurements.push_back(p);
            else
            {                
	        data->imu_measurements.push_back(p);
	        double delta = physical_event - imu_lag_compensated;
		logfile << physical_event << " " << imu_lag_compensated << " " << delta << " " << timespec2double(data->cam_ts) << std::endl;
		if(abs(delta) > 1)
		  fprintf(stderr, "\n * * *\n WARNING: time difference between camera and imu: %g\n * * * \n", delta);
                break;
            }            
        }

	fprintf(stderr, "imu_measurements.size(): %d, rb.size(): %d\n", data->imu_measurements.size(), rb.size());
    } 
    if(data->imu_measurements.size()==1)
      return Error;
    return Ok;
}

bool PluginIMU::event(SDL_Event &event)
{
    bool event_handled = false;   
    return event_handled;
}
