/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Oliver Birbach <oliver.birbach@dfki.de>
 *          Rene Wagner <rene.wagner@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _PLUGIN_OPTIMIZER_H
#define _PLUGIN_OPTIMIZER_H

namespace vslam {
class camera;
class optimizer;
namespace map{
class map;
} // namespace map
} // namespace vslam

#include "processing_plugin.h"
#include "program_config.h"

class PluginOptimizer : public ProcessingPlugin
{
	typedef PluginOptimizer self;
	
protected:
    Buffer * other_buffer;

public:
    PluginOptimizer(std::string _name, Buffer * _buffer, Buffer * _other_buffer,
					vslam::optimizer& opti,
					vslam::map::map &global_map,
					const program_config &pc);
    virtual ~PluginOptimizer();
    virtual Result process(Frame * data);    
    bool event(SDL_Event &event);
    
    EIGEN_MAKE_ALIGNED_OPERATOR_NEW

private:
 	void add_measurement(const vslam::matcher::stereo_match &feature,
 						 const vslam::map::landmark &landmark);
	void add_measurement(const vslam::matcher::mono_feature &feature,
			const vslam::map::landmark &landmark, const vslam::camera & cam, bool is_left);
	void process_known_feature(Frame * data,
							   const vslam::matcher::stereo_match &feature,
							   const vslam::map::landmark &landmark,
							   const float &dist);
	void process_unknown_feature(Frame * data,const vslam::matcher::stereo_match &feature,
								 const float &dist_new, const float &dist_known);
	void process_mono_feature(Frame * data,
			const vslam::matcher::mono_feature &feature,
			const vslam::map::landmark &landmark,
			const float &dist);
	
	bool process_imu(const std::vector<IMURDataPacket> imu_measurements);
	
	void process_landmarks();

	void timing(const char* what, timeutils::timer &t) const;

	vslam::optimizer &opti_;
	vslam::map::map &global_map_;
	int imu_initialization_measurements;
	Eigen::Vector3d gyro_bias;
	Eigen::Vector3d init_gravity;
	
	const program_config &pc;
};

#endif
