/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Oliver Birbach <oliver.birbach@dfki.de>
 *          Rene Wagner <rene.wagner@dfki.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#include "plugin_stereo_match.h"
#include <iomanip>
#include <iostream>


#include "transformations.h"
#include "feature_extraction_config.hpp"

PluginStereoMatch::PluginStereoMatch(std::string _name, Buffer * _buffer, Buffer * _other_buffer,
                                     const program_config &pc)
    : ProcessingPlugin(_name, _buffer), other_buffer(_other_buffer),
	  smatch_(feature_extraction_config::descriptor_size, Transform::cam),
	  pc(pc)
{
}


PluginStereoMatch::~PluginStereoMatch()
{
}


Result PluginStereoMatch::process(Frame * data)
{   

    // Only camera 0 should do some stuff
    if (data->cam_id == 0)
    {
        // Get other Frame from other camera
        Frame *other_data = other_buffer->getPointerAt(other_buffer->getCurrentWriteIdx());

		//std::cerr << "foo: " << data->features.keypoints.size() << " " << data->features.descriptors.size()/128 << " " << other_data->features.keypoints.size() << " " << other_data->descriptors.size()/128 << std::endl;

		timeutils::timer t;
		data->stereo_features.clear();
		data->mono_features.clear();
		
		smatch_(data->features,        // from left frame
				other_data->features,  // from right frame
				data->stereo_features,
				data->mono_features);

		std::cerr << "matching: " << t.elapsed() << " us" << std::endl;
		//std::cerr << "stereo matches: " << data->stereo_features.size() << std::endl;

		feature_extraction_config::feature_extractor extractor;
		long long start_frame = 36;
		long long end_frame = 414;
    
		if(data->cam_id == 0
		   && pc.dump_stereo_matches > 0
		   && data->number >= start_frame
		   && data->number <= end_frame
		   && (data->number - start_frame) % 42 == 0)
		{
			std::cerr << "dumping stereo matches" << std::endl;
			std::stringstream fname;
			fname << pc.dump_dir << "/stereo_match_" << std::setfill('0') << std::setw(4) << data->number << ".dat";
			std::ofstream matches(fname.str().c_str());
			matches << "# " << data->get_filename() << " " 
					<< other_data->get_filename() << " "
					<< extractor.detector_id() << " "
					<< extractor.descriptor_id() << std::endl;
			typedef std::vector<vslam::matcher::stereo_match> stereo_matches;
			const stereo_matches& m = data->stereo_features;
			for(stereo_matches::const_iterator it = m.begin(); it!=m.end(); ++it)
			{
				matches << it->left.pt.x << " " << it->left.pt.y << " " 
						<< it->right.pt.x << " " << it->right.pt.y << " "
						<< it->left.size << " " << it->right.size << std::endl;
			}

  }


    }    

    return Ok;  
}

bool PluginStereoMatch::event(SDL_Event &event)
{
    bool event_handled = false;
    
    return event_handled;
}
