/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Oliver Birbach <oliver.birbach@dfki.de>
 *          Rene Wagner <rene.wagner@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _PLUGIN_VISUALISATION_H
#define _PLUGIN_VISUALISATION_H

#include <boost/shared_ptr.hpp>

#include "processing_plugin.h"
#include "utils/mutex.h"
#include <SDL.h>
#include <string>
#include "program_config.h"
#include "visualisation_window.h"

namespace vslam {
class camera;
namespace map{
class map;
} // namespace map
} // namespace vslam

#include <vslam/camera/camera.hpp>

class PluginVisualisation : public ProcessingPlugin
{
protected:
  Buffer * other_buffer;

  const vslam::map::map &global_map_;

  SDL_Surface *visualisation;

  bool do_draw_video_image;
  bool do_write_rendered_image;
  const std::string &dump_dir;

  int dsf;
  int counter;
  double vis_dsf;
  int left;
  int upper;
  bool drag;
  int drag_start_x;
  int drag_start_y;

  int width;
  int height;
  
  const program_config &pc;

  VisualisationWindow *vw_;
  PluginVisualisation *other_visualisation_;
  
public:
  PluginVisualisation(std::string _name, Buffer * _buffer, Buffer * _other_buffer,
					  const vslam::map::map &global_map,
					  int _width, int _height, int _dsf, const program_config &pc, VisualisationWindow *vw, PluginVisualisation *other_visualisation = 0);
  virtual ~PluginVisualisation();
  virtual Result process(Frame * data);
  bool event(SDL_Event &event);

private:
  typedef vslam::camera::Vector vect;
  typedef vslam::camera::Transform Transform;
  void draw_cross(const cv::Point2f &p, int r, int g, int b, int a);
  void draw_plus(const cv::Point2f &p, int r, int g, int b, int a);
  void draw_line(const cv::Point &p1_, const cv::Point &p2_,
				 int r, int g, int b, int a);
  void draw_text(int x, int y, const std::string &text, int r, int g, int b, int a);
  void draw_filled_circle(const cv::Point &p, int radius, int r, int g, int b, int a);
  void draw_grid(const Transform &world2cam, const vslam::camera &cam);
  void draw_grid_line(const vslam::camera &cam, const vect &p0, const vect &p1, const vect &dir, int y_steps, bool zero_line);
  int calc_grid_line( vslam::camera::image_point coords[], // output array, minimal size [grid_steps+1][2]
  		const vslam::camera &cam, const vect &p0, const vect &dir, int grid_steps);
  template<class Feature>
  unsigned draw_landmark_matches(const std::vector<std::pair<const Feature*, const vslam::map::landmark*> > &map_matches, 
  		cv::KeyPoint Feature::* l_r, const vslam::camera &cam, const Transform &world2cam,
  		int r, int g, int b, int a);
};

#endif
