/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Oliver Birbach <oliver.birbach@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#include "processing_stack.h"
#include <sys/time.h>
#include <iostream>

std::vector<boost::barrier*> ProcessingStack::bar(15);

ProcessingStack::ProcessingStack(std::string _name) : name(_name), timingfile(0) {
}

ProcessingStack::~ProcessingStack() {
}

string ProcessingStack::getName() {
  return name;
}

void ProcessingStack::process(Frame * data) {
  for (unsigned int i=0;i < stack.size();i++) {
    ProcessingPlugin* p(stack[i]);
    p->lock();
    timeutils::timer t;
    Result res = p->process(data);
    if(res != Ok)
   	 std::cerr << "Warning, Processing failed!\n";
    if(timingfile)
   	 *timingfile << t.elapsed() << " ";
    bar[i]->wait();
    if(timingfile)
   	 *timingfile << t.elapsed() << " ";
    p->unlock();
  }
  if(timingfile)
	  *timingfile << std::endl;
}

void ProcessingStack::setup_barriers()
{
    for (unsigned int i=0; i < stack.size(); i++)
    {
        if (bar[i] != 0)
            delete bar[i];
    }    

    for (unsigned int i=0; i < stack.size(); i++)
    {
        bar[i] = new boost::barrier(2);
    }    
}


void ProcessingStack::handle_event ( SDL_Event &event) {
  for (unsigned int i=0; i < stack.size(); i++) {
    ProcessingPlugin* p(stack[i]);
    p->lock();
    bool event_handled = p->event(event);
    p->unlock();
    if (event_handled) break;
  }
}
