/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Oliver Birbach <oliver.birbach@dfki.de>
 *          Rene Wagner <rene.wagner@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _PROCESSING_THREAD_H
#define _PROCESSING_THREAD_H

#include "frame.h"
#include "processing_stack.h"
#include <utils/mutex.h>
#include <utils/thread.h>
#include <camera/generic_camera.h>
#include "image_capture_thread.h"
#include <boost/thread/barrier.hpp>
#include <iostream>

class ProcessingThread : public Thread
{
protected:
  Mutex stack_mutex;
  Mutex capture_mutex;
  boost::mutex stack_executing_mutex;
  ProcessingStack * stack;
  Buffer * rb;
  bool _kill;
  int camId;
  Generic_Camera* cam;
  struct timespec prev;
  images_rb_t& img_rb;
  static boost::barrier bar;
  static int frame_count[2];
   timespec t_0;
  bool t_0_found;
  double camera_lag;
  bool image_rotated;

  std::ostream& timing_file;
public:
  bool init();
  bool stop();
  void join();
  void selectCaptureMethod();

public:
  void setBuffer(Buffer * _rb);
  Buffer * getBuffer() const;
  void setProcessingStack(ProcessingStack * _stack);
  ProcessingStack * getStack() const;
  void kill();

  ProcessingThread(int cam_id, Generic_Camera* cam, images_rb_t&, double _camera_lag, bool, std::ostream& timing_file);
  ~ProcessingThread();

  virtual void run();

private:
  bool keep_running() const;
  bool synchronize();
};
#endif
