/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Oliver Birbach <oliver.birbach@dfki.de>
 *          Rene Wagner <rene.wagner@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _PROGRAM_OPTIONS_H
#define _PROGRAM_OPTIONS_H

#include <boost/program_options.hpp>
namespace po = boost::program_options;

struct program_config
{
  bool nox;
	
  // Camera options 
  std::string left_cam_serial;
  std::string right_cam_serial;
  int image_width;
  int image_height;
  int region_x;
  int region_y;
  int exposure;
  int gain;
  double frame_rate;
  bool rotate_left_cam;
  bool rotate_right_cam;
  int dsf;

  // IMU options
  std::string imu_dev;
  int imu_sample_freq;    

  // Timing options
  double lag_imu;
  double lag_cam;

  // More options
  std::string save_dir; 
  int framebuffer_size; 
  bool play_once;
  unsigned int skip_frames;
  
  //Visualization options:
  bool thick_crosses;
  double grid_stepsize;
  double grid_linewidth;
  double grid_x_min, grid_y_min;
  int grid_x_steps, grid_y_steps;
  int visualize_verbosity;

  // Realtime-behaviour 
  int priority; 
  std::string log_dir; 
  
  // Dump options
  std::string dump_dir;
  int dump_images;
  bool dump_landmarks;
  bool dump_trajectory;
  int dump_stereo_matches;
  int dump_octomap;

  bool compare_trajectory;

  bool extract_features;
  
  int feature_limit;
  unsigned int feature_association_limit;

  // Optimizer options:
  int imu_initialization_frames;
  double start_height;
  int optimization_steps;
  double association_threshold;
  int optimizer_verbosity;
  
  double max_landmark_distance;
  double min_relative_landmark_distance;
  
  double velocity_sigma;

  // dense stereo options
  double min_stereo_range;
  double max_stereo_range;
  int min_disparity;
  int disparity_range;
  bool dump_disparities;

  // calibration options
  int calib_plate_features_vertical;
  int calib_plate_features_horizontal;
  double dist_features;
    
};

void separate_arguments(int argc, char** argv, int & ard_argc, char** ard_argv, int& po_argc, char** po_argv);
void do_program_options(int po_argc, char** po_argv, po::variables_map& vm, program_config &pc);

#endif
