/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Oliver Birbach <oliver.birbach@dfki.de>
 *          Rene Wagner <rene.wagner@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#include "transformations.h"
#include <sstream>

vslam::stereo_camera Transform::cam;
vslam::camera::Transform Transform::left2imu = vslam::pose_as_matrix();

const char Transform::file_header[] = "STEREOCAM";

void Transform::CalibrationToString(std::string &str, bool put_header)
{
	std::ostringstream ss;
	ss.precision(16);
	
	CalibrationToStream(ss, put_header);
	str = ss.str();
}
void Transform::CalibrationToStream(std::ostream &ss, bool put_header)
{
	if(put_header)
		ss << "#" << file_header << "\n";

	cam.get_cam(true) .to_stream(ss);
	cam.get_cam(false).to_stream(ss);

	ss << cam.get_left2right() << std::endl;
	ss << left2imu << std::endl;
}

vslam::camera::file_format Transform::check_format(std::istream &strm)
{
	char c;
	strm >> c;
	if(c != '#')
	{
		strm.putback(c);
		return vslam::camera::legacy;
	}
	std::string fmt;
	strm >> fmt;
	if(fmt == "STEREOCAM")
	{
		return vslam::camera::cv_cam;
	}
	throw "Unknown calibration file format";
	return vslam::camera::legacy;
}


void Transform::StringToCalibration(const std::string &str)
{
	std::istringstream ss(str);
	CalibrationFromStream(ss);
}

void Transform::CalibrationFromStream(std::istream &ss)
{
	vslam::camera::file_format format = check_format(ss);
    cam.set_from_stream(ss, format);

    ss >> left2imu;

    left2imu.normalize();
}

void Transform::CalibrationFromFile(const std::string &f)
{
	std::fstream in;

	in.open(f.c_str(), std::ios ::in);

	in.exceptions(std::ios_base::badbit | std::ios_base::failbit);

	if(in.fail())
		throw "could not read path from file";

	CalibrationFromStream(in);
  
	in.close();
}
