/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Oliver Birbach <oliver.birbach@dfki.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef IMAGE_H
#define IMAGE_H

#include <cstring>
#include "ringbuffer.h"
#include <inttypes.h>

struct Image
{
  Image(): width(640), height(480), bpp(2), data(0), frame_count(0), residual(0)
    {
	data = new unsigned char[width * height * bpp];
    }

  Image(unsigned int width_, unsigned int height_, unsigned int bytes_per_pixel): width(width_), height(height_), bpp(bytes_per_pixel), is_log(false), residual(0)
    {
	data = new unsigned char[width * height * bpp];
    }

    Image( const Image& other): data(0), frame_count(0)
    {
	*this = other;
    }

    Image& operator=(const Image& other)
    {
	if ( this->size() != other.size() )
	{
	    if ( data ) delete[] data;
	    data = new unsigned char[other.width*other.height*other.bpp];
	}
	this->width = other.width;
	this->height = other.height;
	this->bpp = other.bpp;
	this->frame_count = other.frame_count;
        this->is_log = other.is_log;
        this->timestamp = other.timestamp;
        this->os_timestamp = other.os_timestamp;
	this->residual = other.residual;
	memcpy(data, other.data, width * height * bpp);
	return *this;
    }
    ~Image()
    {
      //if ( data ) {
            delete[] data;
	    //  }
        
    }

    unsigned int size() const
    {
	return width*height*bpp;
    }

    unsigned int width;
    unsigned int height;
    unsigned int bpp;
    unsigned char *data;

    struct timespec timestamp;
    unsigned short frame_count;
    bool is_log;
    struct timespec os_timestamp;
    double residual; 
};

typedef Ringbuffer<Image, 10> Image_RB_t;

#endif
