/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Oliver Birbach <oliver.birbach@dfki.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef SSEHELPERS_H_INCLUDED
#define SSEHELPERS_H_INCLUDED

//! Content of an XMM register in various forms
/*! An XMM register can hold 16 bytes, 8 word, 4 dwords, 4 floats or 2 doubles.

    This union is automatically aligned to 16 byte boundary.
 */
union SSEDQWord 
{
  public:
  //! 16 unsigned bytes
  unsigned char  ub[16];
  //! 16 signed bytes 
  signed char    sb[16];    
  //! 8 unsigned words
  unsigned short uw[8];  
  //! 8 signed words
  signed short   sw[8];
  //! 4 unsigned double-words
  unsigned int   ud[4];
  //! 4 signed double-words
  signed int     sd[4];
  //! 4 floating point numbers
  float          f[4];    
  //! 2 double precision floating point numbers
  double         d[2];
  
  //! Set all \c ub to \c x
  void setAllUB (unsigned char x)
  {
    ub[0] = ub[1] = ub[2] = ub[3] = ub[4] = ub[5] = ub[6] = ub[7] = ub[8] = ub[9] = ub[10] = ub[11] = ub[12] = ub[13] = ub[14] = ub[15] = x;
  }
  
  //! Set all \c sb to \c x
  void setAllSB (signed char x)
  {
    sb[0] = sb[1] = sb[2] = sb[3] = sb[4] = sb[5] = sb[6] = sb[7] = sb[8] = sb[9] = sb[10] = sb[11] = sb[12] = sb[13] = sb[14] = sb[15] = x;
  }
  
  //! Set all \c uw to \c x
  void setAllUW (signed short x)
  {
    uw[0] = uw[1] = uw[2] = uw[3] = uw[4] = uw[5] = uw[6] = uw[7] = x;      
  }
  
  //! Set all \c sw to \c x
  void setAllSW (signed short x)
  {
    sw[0] = sw[1] = sw[2] = sw[3] = sw[4] = sw[5] = sw[6] = sw[7] = x;      
  }
  
  //! Set all \c ud to \c x
  void setAllUD (unsigned int x)
  {
    ud[0] = ud[1] = ud[2] = ud[3] = x;            
  }
  
  //! Set all \c sd to \c x
  void setAllSD (signed int x)
  {
    sd[0] = sd[1] = sd[2] = sd[3] = x;            
  }
  
  //! Set all \c f to \c x
  void setAllF (float x)
  {
    f[0] = f[1] = f[2] = f[3] = x;            
  }

  //! Set all \c d to \c x
  void setAllD (double x)
  {
    d[0] = d[1] = x;            
  }
    
}__attribute__((aligned(16)));


//! Fast conversion of float to int with truncation (rounding-->0)
inline int toIntTruncation (float f)
{
  int result;  
  asm (
    "cvttss2si %1, %0   \n\t"
      :"=ir" (result):"m" (f):
    );  
  return result;  
}


//! Fast conversion of float to int with truncation (rounding-->0)
inline int toIntTruncation (double f)
{
  int result;  
  asm (
    "cvttsd2si %1, %0   \n\t"
      :"=ir" (result):"m" (f):
    );  
  return result;  
}

inline bool aligned16 (void *x)
{
  return (((unsigned int)x)&0xf)==0;  
}

#endif
