/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#include <slom/BuildMeasurement.hpp>

#include <slom/m_estimate.hpp>

#include <slom/Estimator.hpp>
#include <slom/CallBack.hpp>
#include <mtk/types/vect.hpp>

#include "random_vector.hh"


typedef MTK::vect<2, double> vec2;
SLOM_RANDOMVAR_ID(vec2)

SLOM_BUILD_MEASUREMENT_JACOBIAN(m_test, 2, ((vec2, est)), ((vec2, real)))
void m_test::eval(MTK::vectview<double, 2> ret, bool num_jacobian) const
{
	if(num_jacobian && 0)
		ret = *est - real;
	else
		SLOM::m_estimate(ret.data(), *est - real, 1);
}



int main()
{
	SLOM::Estimator e;
	SLOM::CallBack cb(e, 100);
	e.setCallBack(cb);
	
	random_vector<> rng;
	
	vec2_id est = e.insertRV(vec2(Eigen::Vector2d(10, 10)));
	
	for(int i=0; i<100; ++i)
	{
		Eigen::Vector2d meas = rng.get_normal<2>();
		e.insertMeasurement(m_test(est,  meas));
		e.insertMeasurement(m_test(est, -meas));
	}
	
	e.insertMeasurement(m_test(est, Eigen::Vector2d(10, 10)));
	
	for(int i=0; i<10; ++i)
	{
		std::cout << "Step " << i << "\t";
		double gain = e.optimizeStep(true);
		(void)gain;
		std::cout << est->transpose() << std::endl;
	}
	
	return 0;
}
