/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */


#include <iostream>
#include <vector>

#include <Eigen/Core>
#include <Eigen/Geometry>
#include <octomap/octomap.h>
#include <vslam/core/optimizer.hpp>

using namespace octomap;

typedef Eigen::Vector3d landmark;
typedef std::vector<landmark> landmark_vector;
typedef vslam::optimizer::Pose pose;

void insert_landmarks(const pose &b2w, const landmark_vector &landmarks, const bool &first,
					  ScanGraph &graph)
{
	Pointcloud *scan = new Pointcloud();
	pose6d scan_pose(point3d(b2w.pos.x(), b2w.pos.y(), b2w.pos.z()),
					 octomath::Quaternion(b2w.orient.w(), b2w.orient.x(), b2w.orient.y(), b2w.orient.z()));
	
	for (landmark_vector::const_iterator i = landmarks.begin();
		 i != landmarks.end(); ++i)
	{
		landmark l_in_b = b2w.orient.conjugate() * (*i - b2w.pos);

		scan->push_back(l_in_b.x(), l_in_b.y(), l_in_b.z());
	}
	graph.addNode(scan, scan_pose);

	if (!first)
		graph.connectPrevious();
}

int main(int argc, char **argv)
{
	landmark_vector landmarks;

	landmarks.push_back(landmark(1., 0., 0.));
	landmarks.push_back(landmark(1., 1., 0.));
	landmarks.push_back(landmark(-1., 0., 0.));
	landmarks.push_back(landmark(-1., 1., 0.));
	landmarks.push_back(landmark(-2., 0., 0.));
	landmarks.push_back(landmark(-2., 1., 0.));
	
	landmarks.push_back(landmark(-1., 0., 1.));
	landmarks.push_back(landmark(-1., 1., 1.));
	landmarks.push_back(landmark(-2., 0., 1.));
	landmarks.push_back(landmark(-2., 1., 1.));
	

	ScanGraph *graph = new ScanGraph();

	pose b2w;
	insert_landmarks(b2w, landmarks, true, *graph);

	b2w.orient = b2w.orient * Eigen::AngleAxis<double>(0.4, Eigen::Vector3d(0., 0., 1.));
	insert_landmarks(b2w, landmarks, false, *graph);

	b2w.pos += Eigen::Vector3d(0., 4., 2.);
	insert_landmarks(b2w, landmarks, false, *graph);

	b2w.orient = b2w.orient * Eigen::AngleAxis<double>(0.4, Eigen::Vector3d(0., 1., 0.));
	insert_landmarks(b2w, landmarks, false, *graph);

	b2w.pos += Eigen::Vector3d(0., -10., 2.);
	insert_landmarks(b2w, landmarks, false, *graph);

	graph->writeBinary("octomap_conversion_test.graph");
	
	return 0;
}
