/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef RANDOM_VECTOR_HH_
#define RANDOM_VECTOR_HH_


#include <boost/random/variate_generator.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/normal_distribution.hpp>
#include <boost/random/uniform_real.hpp>
//#include <boost/random/uniform_on_sphere.hpp>

#include <Eigen/Core>
#include <Eigen/Array>
/**
 * Generate normal or uniform distributed random vectors of arbitrary dimension
 */

template<class RandomGenerator = boost::mt19937>
struct random_vector
{
	random_vector(const RandomGenerator &rng = RandomGenerator()) 
		: rng_(rng), 
		  uniform(rng_, boost::uniform_real<double>(-1.0, 1.0)), 
		  normal(rng_, boost::normal_distribution<double>())
	//, 		  sphere(rng_, boost::uniform_on_sphere<double>())
	{}
	
	template <int dim>
	Eigen::Matrix<double, dim, 1> get_uniform()
	{
		Eigen::Matrix<double, dim, 1> ret;
		for(int i=0; i<dim; ++i){
			ret[i] = uniform();
		}
		return ret;
	}

	template <int dim>
	Eigen::Matrix<double, dim, 1> get_normal()
	{
		Eigen::Matrix<double, dim, 1> ret;
		for(int i=0; i<dim; ++i){
			ret[i] = normal();
		}
		return ret;
	}
	
	template <int dim>
	Eigen::Matrix<double, dim, 1> get_uniform_sphere()
	{
		Eigen::Matrix<double, dim, 1> ret = get_normal<dim>();
		ret.normalize();
		return ret;
	}

	
	RandomGenerator rng_;
	boost::variate_generator<RandomGenerator, boost::uniform_real<double> > uniform;
	boost::variate_generator<RandomGenerator, boost::normal_distribution<double> > normal;
//	boost::variate_generator<RandomGenerator, boost::uniform_on_sphere<double> > sphere;
};



#endif /* RANDOM_VECTOR_HH_ */
