/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef QUALISYS_FROM_FILE_HPP_
#define QUALISYS_FROM_FILE_HPP_

#include <istream>

#include <vslam/core/pose_as_matrix.hpp>

namespace qualisys {

void skip_header(std::istream &inp)
{
	for(int i=0; i<11; ++i)
	{
		// skip first lines of qualisys file
		std::string dummy;
		std::getline(inp, dummy);
	}
}

double readline(std::istream &inp,
                vslam::pose_as_matrix& pose)
{
	double roll, pitch, yaw;
	double res;

	for(int i=0; i<3; ++i)
		inp >> pose.get_translation()(i);

	pose.get_translation() /= 1000; // convert millimeter to meter
	inp >> roll >> pitch >> yaw >> res; // values are ignored

	for(int i=0; i<3; ++i)
		for(int j=0; j<3; ++j)
			inp >> pose.get_rotation()(j,i);

	return res;
}

template<class Derived1, class Derived2>
double readline(std::istream &inp, 
                Eigen::MatrixBase<Derived1> &pos, 
                Eigen::MatrixBase<Derived2> &rotation)
{
	for(int i=0; i<3; ++i)
		inp >> pos[i];
	pos /= 1000; // convert millimeter to meter

	double roll, pitch, yaw;
	double res;
	inp >> roll >> pitch >> yaw >> res; // values are ignored
	
	for(int i=0; i<3; ++i)
		for(int j=0; j<3; ++j)
			inp >> rotation(j,i);

	return res;
}

template<class Derived>
double readline(std::istream &inp,
                Eigen::MatrixBase<Derived> &pos,
                Eigen::Quaterniond& quat)
{
	Eigen::Matrix3d mat;
	double res = readline(inp, pos, mat);
	quat = mat;
	return res;
}


}  // namespace qualisys


#endif /* QUALISYS_FROM_FILE_HPP_ */
