/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Oliver Birbach <oliver.birbach@dfki.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */


#include <fstream>
#include <iostream>
#include "packets/imu_raw_packets.h"

int main(int argc, char **argv)
{
	if(argc < 3){
		std::cerr << "Usage:\n " << argv[0] << " <input> <output> \n";
		return 1;
	}


        std::ifstream in(argv[1], std::ifstream::in | std::ifstream::binary);
        std::ofstream out(argv[2]);
        out << "# timestamp (in s), sample count, acc_x, acc_y, acc_z (all in m/s^2), omega_x, omega_y, omega_z (all in rad/s)" << std::endl;
        while (in.good())
        {
            IMURDataPacket s;
            in.read(reinterpret_cast<char*>(&s), sizeof(IMURDataPacket));

            out.precision(16);
            out << s.m_time.tv_sec + s.m_time.tv_nsec / 1e9 << " " << s.sample_count << " " << s.a[0] << " " << s.a[1] << " " << s.a[2] << " " << s.w[0] << " " << s.w[1] << " " << s.w[2] << std::endl;            
      } 
        in.close();
        out.close();
}
