#!/usr/bin/env ruby

require "pp"

$result_path = "results"
$do_comment = true
$comment_start_sym = '#'


def parse_file( fname, num )
    res = { :far => 0, :close => 0, :unrelated => 0, :unclear => 0 }

    counter = 0
    IO.foreach( fname ) do |l|
      l.chomp!
      next if l =~ /^\#/
      
      case l 
      when "_"
        break
      when "far"
        res[:far] += 1
      when "close"
        res[:close] += 1
      when "unclear"
        res[:unclear] += 1
      when "unrelated"
        res[:unrelated] += 1
      end

      counter += 1
      break if counter == num
    end

    return [counter, res]
end


def count_valid_lines( fname )
  c = 0
  IO.foreach( fname ) do |l|
    next if l =~ /^\#/
    break if l.empty?
    c += 1
  end
  c
end


def parse_directory( path, neednum, maxfiles )
  basefiles_sm = Dir.glob( File.join(path, "stereo_match_????.dat") )
  basefiles_ifm = Dir.glob( File.join(path, "inter_frame_match_????_????.dat") )

  basefiles = []
  if !basefiles_sm.empty?
    basefiles = basefiles_sm
  elsif !basefiles_ifm.empty?
    basefiles = basefiles_ifm
  else
    raise "No basefiles found in dir '#{path}'"
  end

  error = false
  maxlines_per_file = {}
  rfiles = basefiles.collect do |bf|
    f = File.join( File.dirname( bf ), File.basename( bf, ".dat" ) + "_rating.dat" )
    unless File.readable?( f )
      puts "File #{f} must exist"
      error = true
    end
    maxlines_per_file[f] = count_valid_lines( bf )
    f
  end

  rfiles.sort!

  raise "Fix errors first" if error

  result_name = File.split( path )[1]
  raise "Internal error" if result_name.nil? || result_name.empty?
  result_name = File.join( $result_path, "result_" + result_name )

  result_array = []
  error = false

  if maxfiles >= 0 && rfiles.length > maxfiles
    puts "Trim number of files from #{rfiles.length} to #{maxfiles}"
    rfiles.pop while rfiles.length > maxfiles
  end

  rfiles.each do |fname|
    resnum, res = parse_file( fname, neednum )
    if maxlines_per_file[fname] < neednum && maxlines_per_file[fname] == resnum
      puts "WARNING: #{fname} can only have #{maxlines_per_file[fname]} matches but #{neednum} are requested"
    elsif resnum != neednum
      puts "ERROR: #{fname} only has #{resnum} valid matches"
      error = true
    end
    result_array << "#{res[:close]} \t #{res[:far]} \t #{res[:unrelated]} \t #{res[:unclear]}" + ($do_comment ? " \t #{$comment_start_sym} #{fname}" : "")
  end

  unless error
    header = $do_comment ? "#{$comment_start_sym} close  far  unrelated  unclear\n" : ""
    File.open( result_name, 'w') { |f| f.write( header + result_array.join("\n") + "\n" ) }
    puts "Results written to #{result_name}"
  else
    puts "File not written, fix errors first"
  end

end


catch (:arg_fail) do
  throw :arg_fail if ARGV.length != 2 &&  ARGV.length != 3 && ARGV.length != 4

  path = ARGV[0]
  throw :arg_fail unless File.directory?( path )

  num = -1
  begin
    num = Integer( ARGV[1] )
  rescue ArgumentError
    throw :arg_fail
  end

  if ARGV.length >= 3
    $result_path = ARGV[2]
  end

  maxfiles = -1
  if ARGV.length >= 4
    begin
      maxfiles = Integer( ARGV[3] )
    rescue ArgumentError
      throw :arg_fail
    end
  end

  Dir.mkdir( $result_path ) unless File.directory?( $result_path )

  begin
    parse_directory( path, num, maxfiles )
  rescue RuntimeError => err
    puts "Error: #{err.to_s}"
    exit 1
  end

  exit 0
end

puts "Usage: eval.rb <path to data> <number of required valid points per frame> [<result path>] [<max num of files>]"
exit 1



