/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Tobias Hammer <hammer@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#include "helpers.h"

#include <cstdlib>
#include <fstream>


namespace Helper {


bool loadFileToVector( const std::string& filename, std::vector<std::string>& result )
{
    std::ifstream fileobj( filename.c_str(), std::ios_base::in );
    std::string line;

    if ( !fileobj || !fileobj.is_open() )
        return false;

    result.clear();
    while ( getline( fileobj, line ) )
        result.push_back( line );

    return true;
}


std::vector<std::string> splitStringToVector( const std::string& s, int tok_count, char sep )
{
    size_t pos, start = 0;
    std::vector<std::string> res;

    while ( (pos = s.find_first_of( sep, start )) != std::string::npos )
    {
        res.push_back( s.substr( start, pos-start ) );
        start = pos+1;

        if ( tok_count >= 0 && res.size() >= (size_t)tok_count )
            break;
    }
    res.push_back( s.substr( start ) );

    if ( tok_count >= 0 && res.size() < (size_t)tok_count )
        throw "Not enough matches in string";

    return res;
}


std::vector<double> vector2double( const std::vector<std::string>& vec )
{
    std::vector<double> res;
    for ( size_t i = 0; i < vec.size(); ++i )
    {
        char *endptr;
        res.push_back( strtod( vec[i].c_str(), &endptr ) );
        if ( *endptr != '\0' )
            throw "Conversion from string to double failed";
    }
    return res;
}


std::vector<int> vector2int( const std::vector<std::string>& vec )
{
    std::vector<int> res;
    for ( size_t i = 0; i < vec.size(); ++i )
    {
        char *endptr;
        res.push_back( strtol( vec[i].c_str(), &endptr, 10 ) );
        if ( *endptr != '\0' )
            throw "Conversion from string to int failed";
    }
    return res;
}


}
