/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Tobias Hammer <hammer@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef IMAGEVIEWER_H
#define IMAGEVIEWER_H


#include <string>
#include <opencv/cv.h>


// forward declaration
class StereoImageMatches;
struct StereoPoint;
namespace vslam{ class stereo_camera; }


class ImageViewer
{
public:
    ImageViewer( const StereoImageMatches& sim, const std::string& workdir,
                 int windowWidth, const std::string& windowName, const vslam::stereo_camera& camera );
    ~ImageViewer();

    void show();

    float getScale() const;
    void setActiveMatch( size_t point_idx );

    inline bool getShowCircles() const;
    inline void setShowCircles( bool active );
    inline bool getShowEpilines() const;
    inline void setShowEpilines( bool active );
    inline bool getShowMinimal() const;
    inline void setShowMinimal( bool active );
    inline bool getShowLargeX() const;
    inline void setShowLargeX( bool active );


private:

    void checkAndInit();
    IplImage* loadImage( std::string filename ) const;
    void newMultiImage();
    void drawMatches();
    void drawEpilines();
    void drawCircles( const CvPoint& mid );
    void drawCross( IplImage *img, int x, int y, CvScalar color, int thickness ) const;

    const StereoImageMatches& m_sim;
    std::string m_workdir;

    IplImage *m_leftImg,
             *m_rightImg,
             *m_multiImg;
    const vslam::stereo_camera& m_camera;

    float m_scale;
    int m_windowWidth;
    int m_multiWidth,
        m_multiHeight;
    int m_crossSize;
    size_t m_activeMatchIdx;
    bool m_showCircles;
    bool m_showEpilines;
    bool m_minimalDisplay;
    bool m_largeX;

    std::string m_windowName;
};


// *** inline methods ***

bool ImageViewer::getShowCircles() const
{
    return m_showCircles;
}

void ImageViewer::setShowCircles( bool active )
{
    m_showCircles = active;
}

bool ImageViewer::getShowEpilines() const
{
    return m_showEpilines;
}

void ImageViewer::setShowEpilines( bool active )
{
    m_showEpilines = active;
}

bool ImageViewer::getShowMinimal() const
{
    return m_minimalDisplay;
}

void ImageViewer::setShowMinimal( bool active )
{
    m_minimalDisplay = active;
}

bool ImageViewer::getShowLargeX() const
{
    return m_largeX;
}

void ImageViewer::setShowLargeX( bool active )
{
    m_largeX = active;
}



#endif // IMAGEVIEWER_H
