/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Tobias Hammer <hammer@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */


#include <string>
#include <iostream>

#include <opencv/cv.h>
#include <opencv/highgui.h>

#include "stereomatchgui.h"



void print_usage( const char* pname )
{
    std::cout << "Usage: " << pname << " <path>" << std::endl;
    std::cout << std::endl;
    StereoMatchGui::printKeyHints();
}


int main( int argc, char** argv )
{
    cvInitSystem( argc, argv );
    setlocale(LC_ALL, "C");

    if ( argc != 2 )
    {
        print_usage( argv[0] );
        return 0;
    }

    try {

        StereoMatchGui smg( argv[1], 1500 );    // 1500: image width
        smg.run();

    }
    catch ( const char* err )
    {
        std::cerr << "UNCAUGHT EXCEPTION (char*): " << err << std::endl;
        return 1;
    }
    catch ( const std::string &err )
    {
        std::cerr << "UNCAUGHT EXCEPTION (string): " << err << std::endl;
        return 1;
    }
    catch ( const std::exception &e )
    {
        std::cerr << "UNCAUGHT EXCEPTION (std::exc): " << e.what() << std::endl;
        return 1;
    }
    catch ( ... )
    {
        std::cerr << "UNCAUGHT EXCEPTION (unknown)" << std::endl;
        return 1;
    }

    return 0;
}

