/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Tobias Hammer <hammer@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef STEREOIMAGEMATCHES_H
#define STEREOIMAGEMATCHES_H


#include <vector>
#include <string>
#include <opencv/cv.h>


struct StereoPoint
{
    StereoPoint( float lx, float ly, float rx, float ry, int size_left_, int size_right_ )
    { l.x = lx; l.y = ly; r.x = rx; r.y = ry; size_left = size_left_; size_right = size_right_; }

    CvPoint2D32f l, r;
    int size_left, size_right;
};


class StereoImageMatches
{
public:
    StereoImageMatches( const std::string& matchfile );
    ~StereoImageMatches();

    void loadFile( const std::string& matchfile );

    std::string lImageName() const;
    std::string rImageName() const;

    size_t matchCount() const;
    StereoPoint getSPoint( size_t pos ) const;

    enum {
        R_CLOSE    = 0,
        R_FAR,
        R_UNRELATED,
        R_UNCLEAR,
        R_SPURIOUS,
        R_UNRATED,
        _R_MAX_
    };
    void setRating( size_t pos, int rating );
    int  getRating( size_t pos ) const;
    int  getNumUnrated() const;
    std::string getRatingAsString( size_t pos ) const;

    void writeRating() const;

    static float getMatchRadius( int type );

private:

    std::string constructRatingfileName() const;
    void tryLoadRating();

    std::string m_matchfile,
                m_limage,
                m_rimage;

    std::vector<StereoPoint> m_spoints;
    std::vector<int> m_rating;
    std::vector<std::string> m_fileTextMap;
};




#endif // STEREOIMAGEMATCHES_H
