/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Tobias Hammer <hammer@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef STEREOMATCHGUI_H
#define STEREOMATCHGUI_H

#include <string>
#include <vector>
#include <opencv/cv.h>

/*#define NO_MTK_SUPPORT 1
#include <vslam/camera/cv_camera.hpp>
*/

// forward declarations
class StereoImageMatches;


class StereoMatchGui
{
public:
    StereoMatchGui( const std::string& path, int maxWidth = 1000 );
    ~StereoMatchGui();

    void run();
    static void printKeyHints();

private:
    enum { FIRST, LAST, NEXT, PREV, QUIT };
    int handleSingleImagePair( size_t idx );

    std::string m_path;
    std::vector<std::string> m_matchfiles;
    int m_maxWidth;
    int m_crossSize;
    int m_maxRating;
    bool m_skipMode;
    bool m_stereoMode;

    std::string m_camfilename;

    /*
      Ratings:
        '1'     0%sz - 25%sz
        '2'     25%sz - 50%sz
        '3'     50%sz - 75%sz
        '4'     75%sz - 100%sz
        '5'     100%sz- ...
        'u'     unclear
        'p'     spurious
        ' '     skip rating

      Keys
        left/right arrow        match nav
        page up/down            image nav
        esc/q                   quit

        's'         skip mode
        'c'         toggle circle
        'e'         toggle epilines
        'h'         print help on console
    */

    // stl helper functors
    struct pathname_of;
    struct pathname_filter_neg;
    struct pathname_filter;
};



#endif // STEREOMATCHGUI_H
