/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Rene Wagner <rene.wagner@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#include "calibrator.hpp"

#include <slom/m_estimate.hpp>

namespace vslam {

void calibrator::acc_measurement::eval(MTK::vectview<double, 3> ret) const
{
	vec3 acc_world = pose_->get_rotation() * (left2imu->conjugate() * (acc - *acc_offset));
	ret = (acc_world - Eigen::Vector3d(0, 0, *g)) / acc_sigma;
}



void calibrator::point_measurement::eval(MTK::vectview<double, 4> ret) const
{
	vec3 point_in_left = pose_->inverse() * world_;
	camera::image_point left, right;
	cam_->left2images(point_in_left, left, right);
	ret(0) = (left - left_).x;
	ret(1) = (left - left_).y;
	ret(2) = (right - right_).x;
	ret(3) = (right - right_).y;
	ret /= sigma_;
}

void calibrator::groundtruth_measurement::eval(MTK::vectview<double, 6> ret, bool num_jacobian) const
{
	const pose& left2checker = *pose_;
	pose star2world_est = *checker2world * left2checker * *star2left;
	
	star2world_est.boxminus(ret, star2world);
	
	ret /= sigma;
	if(!num_jacobian)
		SLOM::m_estimate(ret.data(), ret, 1);
}


} // namespace vslam

