/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Rene Wagner <rene.wagner@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __FAST_DETECTOR_H__
#define __FAST_DETECTOR_H__

// FAST feature detector

class FastDetector
{
	enum
	{
		levels = 4
	};
	
public:
	FastDetector(const cv::Size &size,
				 bool nonmax_suppression)
		: nonmax_suppression_(nonmax_suppression)
	{
		using namespace cv;
		Size s = size;
		
		for (int i = 0; i < levels; ++i)
		{
			pyramid[i] = Mat(s, CV_8U);
			s = Size(0.5 * s.width, 0.5 * s.height);
		}
	}

	void operator()(const cv::Mat& image, std::vector<cv::KeyPoint>& keypoints) const
	{
		using namespace cv;

		int th[] = {5, 5, 5, 5};
		assert(sizeof(th)/sizeof(int) == levels);

		float s = 6.f;

		for (int i = 0; i < levels; ++i)
		{			
			resize(image, pyramid[i], pyramid[i].size());

			std::vector<KeyPoint> kp;
			FAST(pyramid[i], kp,
				 th[i],
				 nonmax_suppression_);

			for (std::vector<KeyPoint>::iterator k = kp.begin();
				 k != kp.end(); ++k)
			{
				//std::cout << "score: " << k->response << std::endl;
				k->size = s;
				keypoints.push_back(*k);
			}

			s *= 2.f;
		}
	}

private:
	mutable cv::Mat pyramid[levels];

	bool nonmax_suppression_;
};

#endif // __FAST_DETECTOR_H__
