/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Rene Wagner <rene.wagner@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __VSLAM_FEATURES_DISTANCE_EUCLIDEAN_FLOAT_HPP__
#define __VSLAM_FEATURES_DISTANCE_EUCLIDEAN_FLOAT_HPP__

#include <numeric>
#include "util/squared_diff.hpp"

namespace vslam {
namespace features {

class distance_euclidean_float
{
public:
	typedef float val_type;
	typedef float dist_type;
	
	static dist_type convert_one(const float &val)
	{
		return val;
	}

	static val_type convert_back(const dist_type &val)
	{
		return val;
	}

	template<typename Iter>
	static dist_type compute(const Iter &a_begin, const Iter &a_end, const Iter &b_begin)
	{
		dist_type dist = 0;
		dist = std::inner_product(a_begin, a_end, b_begin, dist, std::plus<dist_type>(), util::squared_diff);
		return sqrt(dist);
	}
};

} // namespace features
} // namespace vslam

#endif // __VSLAM_FEATURES_DISTANCE_EUCLIDEAN_FLOAT_HPP__
