/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Rene Wagner <rene.wagner@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __VSLAM_FEATURES_DISTANCE_SSE_HPP__
#define __VSLAM_FEATURES_DISTANCE_SSE_HPP__

#include <stdint.h>

#include <emmintrin.h>

namespace vslam {
namespace features {

class distance_sse
{
public:
	typedef uint8_t val_type;
	typedef int dist_type;

	static val_type convert_one(const float &val)
	{
		//assert (val <= 1.f);
		//assert (-1.f <= val);
		return 128.f * val + 128.f;
	}
	
	static float convert_back(const dist_type &val)
	{
		return (1.f/128) * (float(val) - 128.f);
	}

	template<typename Iter>
	static dist_type compute(const Iter &a_begin, const Iter &a_end, const Iter &b_begin)
	{
		int size = a_end - a_begin;
		int nLoop = size/(sizeof(__m128i)/sizeof(val_type));

		__m128i m1, m2;

		__m128i* pSrc1 = (__m128i*) &*a_begin;
		__m128i* pSrc2 = (__m128i*) &*b_begin;

		m1 = _mm_setzero_si128();

		for ( int i = 0; i < nLoop; ++i)
		{
			m2 = _mm_sad_epu8(*pSrc2, *pSrc1);

			m1 = _mm_add_epi16(m1, m2);

			++pSrc1;
			++pSrc2;
		}

		union u
		{
			__m128i m;
			uint16_t v[sizeof(__m128i)/sizeof(uint16_t)];
		};
		
		__m128i result = m1;
		u *p = (u*) &result;
		return p->v[0] + p->v[4];
	}
};

} // namespace features
} // namespace vslam
	
#endif // __VSLAM_FEATURES_DISTANCE_SSE_HPP__
