/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Rene Wagner <rene.wagner@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __VSLAM_FEATURES_DISTANCE_SSE_COMP_HPP__
#define __VSLAM_FEATURES_DISTANCE_SSE_COMP_HPP__

#include "distance_sad_int.hpp"
#include "distance_sse.hpp"

namespace vslam {
namespace features {

// Checks SSE result against regular integer arithmetic result
class distance_sse_comp
{
public:
	typedef uint8_t val_type;
	typedef int dist_type;

private:

public:
	static val_type convert_one(const float &val)
	{
		//assert (val <= 1.f);
		//assert (-1.f <= val);
		return 128. * val + 128.;
	}

	template<typename Iter>
	static dist_type compute(const Iter &a_begin, const Iter &a_end, const Iter &b_begin)
	{
		dist_type sse = distance_sse::compute(a_begin, a_end, b_begin);
		dist_type ref = distance_sad_int::compute(a_begin, a_end, b_begin);

		assert(sse == ref);
		
		return sse;
	}
};

} // namespace features
} // namespace vslam
	
#endif // __VSLAM_FEATURES_DISTANCE_SSE_COMP_HPP__
