/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Rene Wagner <rene.wagner@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __VSLAM_MAP_FEATURE_MAP_HPP__
#define __VSLAM_MAP_FEATURE_MAP_HPP__

#include <boost/bind.hpp>

#include "../features/distance_sse.hpp"
#include "../matcher/match_utils.hpp"
#include "../matcher/fwd_rev_graph_matcher.hpp"

namespace vslam {
namespace map {

class feature_map
{
	
public:
	typedef std::vector<cv::KeyPoint> keypoints_vector;
	typedef std::vector<float> descriptors_vector;
	typedef matcher::generic_match<matcher::stereo_match, matcher::stereo_match> inter_frame_match;

	feature_map(int descriptor_size)
		: descriptor_size_(descriptor_size)
	{}
	
	void nearest_neighbors(const std::vector<matcher::stereo_match> &stereo_matches,
								std::vector<inter_frame_match> &left_matches) const
	{
		nearest_neighbors_mono(true, stereo_matches, left_matches);
	}
	
	int size() const
	{
		return last_frame_matches.size();
	}

	void clear()
	{
		last_frame_matches.clear();
	}

	void push_back(const matcher::stereo_match &m)
	{
		last_frame_matches.push_back(m);
	}
	

private:
	void nearest_neighbors_mono(bool ,
								const std::vector<matcher::stereo_match> &stereo_matches,
								std::vector<inter_frame_match> &matches) const
	{

		fwd_rev_graph_matcher<vslam::features::distance_sse, matcher::stereo_match, matcher::stereo_match> m;
		
		m(descriptor_size_, 
		  stereo_matches,
		  last_frame_matches,
		  boost::bind(matcher::internal::append_match<matcher::stereo_match, matcher::stereo_match>, boost::ref(matches), _1, _2, _3));
				
	}
	int descriptor_size_;
	std::vector<matcher::stereo_match> last_frame_matches;
//	keypoints_vector keypoints_left_, keypoints_right_;
//	descriptors_vector signatures_;
};

} // namespace map
} // namespace vslam

#endif // __VSLAM_MAP_FEATURE_MAP_HPP__
