/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Rene Wagner <rene.wagner@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __VSLAM_MAP_LANDMARK_HPP__
#define __VSLAM_MAP_LANDMARK_HPP__

#include <algorithm>
#include <vector>

#include "../slom_variables.hpp"


namespace vslam {
namespace map {
		
class landmark
{
public:
	typedef std::vector<float> signature_vector;
	
	landmark(landmark_id pos,
			 const signature_vector &sig)
		: position(pos),
		  signature(sig.size()),
		  num_descriptors(1)
	{
		std::copy(sig.begin(), sig.end(), signature.begin());
	}

	landmark(landmark_id pos,
			 signature_vector::const_iterator sig_begin, signature_vector::const_iterator sig_end)
		: position(pos),
		  signature(sig_end - sig_begin),
		  num_descriptors(1)
	{
		std::copy(sig_begin, sig_end, signature.begin());
	}
	
	void update(signature_vector::const_iterator sig_begin, signature_vector::const_iterator sig_end)
	{
		assert(unsigned(sig_end - sig_begin) == signature.size());
		signature_vector::const_iterator src = sig_begin;
		
		for (signature_vector::iterator dst = signature.begin();
			 dst != signature.end(); ++src, ++dst)
		{
			*dst = (num_descriptors * *dst + *src)/double(num_descriptors + 1);
		}
		++num_descriptors;
	}

	const signature_vector &sig() const
	{
		return signature;
	}

	landmark_id position;
	signature_vector signature;
	
private:
	int num_descriptors;
};

} // namespace map
} // namespace vslam

#endif // __VSLAM_MAP_LANDMARK_HPP__
