/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Rene Wagner <rene.wagner@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __VSLAM_MAP_MAP_HPP__
#define __VSLAM_MAP_MAP_HPP__

#include <deque>

#include <boost/bind.hpp>

#include "../features/distance_sse.hpp"
#include "../matcher/fwd_rev_graph_matcher.hpp"
#include "../matcher/match.hpp"
#include "landmark.hpp"

namespace vslam {
namespace map {

class map
{
	typedef map self;
	
public:
	typedef std::deque<landmark> landmarks_list;
	map(int descriptor_size)
		: descriptor_size_(descriptor_size)
	{}
	
	void clear()
	{
		landmarks_.clear();
	}

	landmark &push_back(const landmark &m)
	{
		landmarks_.push_back(m);
		return landmarks_.back();
	}

	template<typename Feature,
			 typename FoundFunc>
	void nearest_neighbors(const std::vector<Feature> &features,
						   FoundFunc f) const
	{
		fwd_rev_graph_matcher<vslam::features::distance_sse, Feature, landmark> m;

		m(descriptor_size_,
		  features, landmarks_,
		  f);
	}
	
	template<typename Feature,
			 typename FoundFunc,
			 typename NotFoundFunc>
	void nearest_neighbors(const std::vector<Feature> &features,
						   FoundFunc f, NotFoundFunc g) const
	{
		fwd_rev_graph_matcher<vslam::features::distance_sse, Feature, landmark> m;

		m(descriptor_size_,
		  features, landmarks_,
		  f, g,
		  boost::bind(&self::ignore, this, _1, _2, _3));
	}

private:
	void ignore(const landmark &l, vslam::features::distance_sse::dist_type d1, vslam::features::distance_sse::dist_type d2) const
	{}

public:
	const landmark *closest_landmark(const Eigen::Vector3d &p, float *distance) const
	{
		const landmark *closest = 0;
		float dist = 1e9;
		
		for(landmarks_list::const_iterator i = landmarks_.begin();
			i != landmarks_.end(); ++i)
		{
			float d = (p - *i->position).norm();
			if (d < dist)
			{
				closest = &*i;
				dist = d;
			}
		}

		if (distance)
			*distance = dist;
		
		return closest;
	}
	
	const landmarks_list& get_landmarks() const
	{
		return landmarks_;
	}

private:
	int descriptor_size_;
	landmarks_list landmarks_;
};

} // namespace map
} // namespace vslam



#endif // __VSLAM_MAP_MAP_HPP__
