/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Rene Wagner <rene.wagner@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __VSLAM_MATCHER_MATCH_HPP__
#define __VSLAM_MATCHER_MATCH_HPP__

#include <Eigen/Core>
#include <cv.h>

#include "../util/const_sub_container.hpp"

namespace vslam {
namespace matcher {

/**
 * Container view, which generates the mean values of two containers.
 */
template<class container>
class mean_value_container________DEPRECATED
{
	typedef typename container::const_iterator const_iterator_;
public:
	typedef typename container::value_type value_type;
	typedef typename container::size_type size_type;
	
	class const_iterator
	{
		const_iterator_ a, b;
		
	public:
		value_type operator*() const
		{
			return (a + b) / value_type(2);
		}
		
		const_iterator& operator++()
		{
			++a; ++b;
			return *this;
		}
		bool operator==(const const_iterator &other) const
		{
			return a == other.a && b == other.b;
		}
		bool operator!=(const const_iterator &other) const
		{
			return !operator==(other);
		}
	};
};

struct stereo_match
{
	typedef std::vector<float> desc_container;
	stereo_match(int descriptor_size,
				 const cv::KeyPoint &l, desc_container::const_iterator d_l,
				 const cv::KeyPoint &r, desc_container::const_iterator d_r,
				 const float &d, const Eigen::Vector3d &p)
		: left(l),
		  right(r),
		  dist(d),
		  world_point_(p)
	{
		signature.reserve(descriptor_size);
		for(int i = 0; i < descriptor_size; ++i)
		{
			signature.push_back(.5f*(*d_l + *d_r));
			++d_l; ++d_r;
		}
	}

	void set_world_point_in_left_camera(const Eigen::Vector3d &p)
	{
		world_point_ = p;
	}

	const Eigen::Vector3d &world_point_in_left_camera() const
	{
		return world_point_;
	}

	const desc_container &sig() const
	{
		return signature;
	}
	
	cv::KeyPoint left, right;
	desc_container signature;
	float dist;

private:
	Eigen::Vector3d world_point_;
};

struct mono_feature
{
	typedef const_sub_container<std::vector<float> > signature_container;
	
	mono_feature(int descriptor_size, const cv::KeyPoint &kp, std::vector<float>::const_iterator desc,
				 const float &dist_l,const float &dist_r)
		: keypoint(kp),
		  signature(desc, desc + descriptor_size),
		  dist_l(dist_l),
		  dist_r(dist_r)
	{}

	const signature_container &sig() const
	{
		return signature;
	}
	
	cv::KeyPoint keypoint;
	signature_container signature;
	float dist_l, dist_r;
};

} // namespace matcher
} // namespace vslam

#endif // __VSLAM_MATCHER_MATCH_HPP__
