/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Rene Wagner <rene.wagner@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __VSLAM_MATCHER_MATCH_GRAPH_NODE_HPP__
#define __VSLAM_MATCHER_MATCH_GRAPH_NODE_HPP__

namespace vslam {
namespace internal {

template<typename A,
		 typename B,
		 typename dist_type>
class match_graph_node
{
	typedef tbb::spin_mutex node_mutex;
	node_mutex mtx_;
		
public:
	typedef std::vector<float> desc_vector;

	match_graph_node(const A *self_
			   )
		: self(self_),
		  nn(0),
		  nn_dist(1000)
	{}

	match_graph_node()
		: self(0),
		  nn(0),
		  nn_dist(1000)
	{}

	void update_unsafe(match_graph_node<B,A,dist_type> *other, dist_type dist)
	{
		if (dist < nn_dist || (dist == nn_dist && other < nn) )
		{
			nn = other;
			nn_dist = dist;
		}
	}
		
	void update(match_graph_node<B,A,dist_type> *other, dist_type dist)
	{
		node_mutex::scoped_lock(mtx_);

		update_unsafe(other, dist);
	}

	const A *self;
	match_graph_node<B,A,dist_type> *nn;
	dist_type nn_dist;
};

} // namespace internal
} // namespace vslam

#endif // __VSLAM_MATCHER_MATCH_GRAPH_NODE_HPP__
