/*
 *  Copyright (c) 2010--2011, Universitaet Bremen and DFKI GmbH
 *  All rights reserved.
 *
 *  Author: Rene Wagner <rene.wagner@dfki.de>
 *          Christoph Hertzberg <chtz@informatik.uni-bremen.de>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Universitaet Bremen nor the DFKI GmbH 
 *     nor the names of its contributors may be used to endorse or 
 *     promote products derived from this software without specific 
 *     prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __VSLAM_MATCHER_MATCH_UTILS_HPP__
#define __VSLAM_MATCHER_MATCH_UTILS_HPP__

#include "match.hpp"
#include "../features/feature_points.hpp"

namespace vslam {
namespace matcher {
template<typename A,
		 typename B>
struct generic_match
{
	typedef std::vector<float> desc_vector;

	generic_match(const A *l, 
				  const B *r, 
				  const float &d)
		: left(l),
		  right(r),
		  dist(d)
	{}
	
	const A *left;
	const B *right;
	float dist;
};

namespace internal {
template<class Left, class Right>
static
void append_match(std::vector<generic_match<Left, Right> > &matches,
				  const Left &left, 
				  const Right &right,
				  const float &dist)
{
	//std::cout << "match found: " << left.pt.x << "," << left.pt.y << " --> " << right.pt.x << "," << right.pt.y << std::endl;
	matches.push_back(generic_match<Left, Right>(&left, &right, dist));
}

static
void nop_match(const cv::KeyPoint &left, std::vector<float>::const_iterator d_l,
			   const cv::KeyPoint &right, std::vector<float>::const_iterator d_r,
			   const float &dist)
{
}
template<class Type>
static void nop_match(const Type&, const float&, const float&)
{
}

} // namespace internal
} // namespace matcher
} // namespace vslam
	
#endif // __VSLAM_MATCHER_MATCH_UTILS_HPP__
